import React, { useState } from 'react';

import { ThemeProvider, createTheme } from '@mui/material/styles';

import { Stack, Box, TextField, IconButton, Typography, Slider } from '@mui/material';

import SendRoundedIcon from '@mui/icons-material/SendRounded';
import CloseRoundedIcon from '@mui/icons-material/CloseRounded';
import OpenInFullIcon from '@mui/icons-material/OpenInFull';
import CloseFullscreenIcon from '@mui/icons-material/CloseFullscreen';
import CachedRoundedIcon from '@mui/icons-material/CachedRounded'; // Refresh Icon, while generating a response

import chatbot_icon from "../../assets/chatbot_icon.svg";
import user_icon from '../../assets/user_icon.png';

import { 
    ChatbotWrapper,
    ChatbotMessageContainer,
    ChatbotHeader,
    ChatbotBotBubble,
    ChatbotUserBubble,
    ChatbotInputMessageContainer,
 } from './CustomStyledComponents';

import './Chatbot.css';

const theme = createTheme({
    text: {
        fontFamily: 'monospace',
    },
    palette: {
      background: '#F5F5F5',
      headerBgColor: '#14A3C7',
      headerFontColor: '#FFFFFF',
      botBubbleBgColor: '#14A3C7',
      botBubbleFontColor: '#FFFFFF',
      userBubbleBgColor: '#FFFFFF',
      userBubbleFontColor: '#171717',
      msgInputBgColor: '#FFFFFF',
    },
});

export const Chatbot = (props) => {
    const {
        // Cconfigarable UI Settings
        headerLabel,
        defaultBotMessageValue,
        inputLabel,
        // customTheme,
        // botIcon,
        // userIcon,
        // closeIcon,
        // sendIcon,

        // Event Handlers
    } = props;

    const [openContainer, setOpenContainer] = useState(false);
    const [expand, setExpand] = useState(false);

    const [inputMsg, setInputMsg] = useState('');
    const [botMessages, setBotMessages] = useState([
        {
            message: process.env.LABEL_DEFAUL_BOT_MESSAGE ? process.env.LABEL_DEFAUL_BOT_MESSAGE : 'Hey, How can I help you today?',
            sender: 'chatBot',
        },
    ]);
    const [userMessages, setUserMessages] = useState([]);
    const [generatingRes, setGeneratingRes] = useState(false);
    const [stopgeneratingRes, setStopGeneratingRes] = useState(false);

    // For Input Range Slider Labels
    const marks = [
        {
          value: 0,
          label: process.env.LABEL_SLIDER_MINIMUM,
        },
        {
          value: 100,
          label: process.env.LABEL_SLIDER_MAXIMUM,
        },
      ];

    const handleInputMsg = (e) => {
        setInputMsg(e.target.value);
    };

    const handleSend = (e) => {
        const newUserMsg = {
            message: inputMsg,
            sender: 'user',
        };

        const newMessages = [...userMessages,  newUserMsg]; // old msg + new msg

        // Update message states
        setUserMessages(newMessages);
        // Reset Input Field
        setInputMsg('');
        setGeneratingRes(true);

        // Process msg to API (send and see the response)

        // 3s timer for showing generating res.
        setTimeout(() => {
            setGeneratingRes(false);
            setStopGeneratingRes(true);
        }, 3000);
    };

    return(
        <ThemeProvider theme={theme}>
            <ChatbotWrapper className='root_chatbot_wrapper'>
                {/* Chatbot-Btn */}
                <Box
                    component="button"
                    className='chatbot_btn'
                    onClick={() => setOpenContainer(!openContainer)}
                    aria-label='chatbot button'
                >
                    <Box
                        component="img"
                        src={chatbot_icon}
                    />
                </Box>

                {/* Chatbot-Container */}
                <ChatbotMessageContainer className={`chatbot_message_container ${openContainer && 'show'} ${expand && 'expand'}`}>
                    {/* Chatbot-Header */}
                    <ChatbotHeader className='chatbot_header'>
                        <Box component="p">{process.env.LABEL_CHATBOT_HEADER ? process.env.LABEL_CHATBOT_HEADER : 'Ask me Anything'}</Box>
                        <Box 
                            component="button"
                            className='chatbot_expand_btn'
                            onClick={() => setExpand(!expand)}
                        >
                            {!expand ? <OpenInFullIcon fontSize="small" htmlColor="#fff" /> : <CloseFullscreenIcon fontSize="small" htmlColor="#fff" />}
                        </Box>
                        <Box 
                            component="button"
                            className='chatbot_close_btn'
                            onClick={() => setOpenContainer(!openContainer)}
                        >
                            <CloseRoundedIcon htmlColor="#fff" />
                        </Box>
                    </ChatbotHeader>

                    {/* Slider */}
                    <Box
                        component="div"
                        className='chatbot_slider'
                    >
                        <Typography className='chatbot_slider_header'>Set tone of responses</Typography>
                        <Slider
                            size="medium"
                            defaultValue={50}
                            aria-label="Slider"
                            valueLabelDisplay="auto"
                            marks={marks}
                            className='chatbot_slider_input'
                        />
                    </Box>

                    {/* Chatbot-Messages */}
                    <Box
                        component="div"
                        className='chatbot_chatMessage_wrapper'
                    >
                        {/* Bot Message Bubble */}
                        <Stack
                            alignItems='flex-start'
                            direction='row'
                            className='chatbot_messageContainer_botMessage'
                        >
                            <Box
                                className='chatbot_messageContainer_botIcon'
                                component="img"
                                src={chatbot_icon}
                            />

                            <Box>
                                {botMessages.map((cbMsg, i) => {
                                    return<ChatbotBotBubble key={i} className='chat_msg'>{cbMsg.message}</ChatbotBotBubble>;
                                })}
                            </Box>
                        </Stack>

                        {/* User Message Bubble */}
                        {userMessages.length ? <Stack
                            alignItems='flex-start'
                            direction='row'
                            className='chatbot_messageContainer_UserMessage'
                        >
                            <Box>
                                {userMessages?.map((umsg, i) => {
                                    return <ChatbotUserBubble key={i} className='chat_msg'>{umsg.message}</ChatbotUserBubble>;
                                })}
                            </Box>
                            <Box
                                className='chatbot_messageContainer_userIcon'
                                component="img"
                                src={user_icon}
                            />
                        </Stack> : ''}

                        {/* Generating Response Text */}
                        {generatingRes ? 
                            <Box
                                component="div"
                                className='chatbot_messageContainer_ResponseMsg'
                            >
                                <Typography className='chat_msg'>
                                    <CachedRoundedIcon className='refresh_icon' />
                                    Generating the response
                                </Typography>
                            </Box> : ''}
                        
                        {/* Stop Generating Response */}
                        {stopgeneratingRes ?
                            <Box
                                component="div"
                                className='chatbot_messageContainer_ResponseMsg stop'
                                onClick={() => setStopGeneratingRes(false)}
                            >
                                <Typography className='chat_msg'>
                                    <CloseRoundedIcon fontSize="small" />
                                    Stop generating the response
                                </Typography>
                            </Box> : ''}
                    </Box>

                    {/* Chatbot-Message-Input */}
                    <ChatbotInputMessageContainer className='chatbot_messageInput_container'>
                        <TextField
                            id="standard-basic"
                            placeholder={process.env.LABEL_INPUT_LABEL ? process.env.LABEL_INPUT_LABEL : "Type Something..."}
                            variant="standard"
                            className='chatbot_messageInput_textField'
                            value={inputMsg}
                            onChange={handleInputMsg}
                        />
                        <IconButton
                            aria-label="send"
                            className='chatbot_message_sendIcon'
                            onClick={(e) => handleSend(e)}
                        >
                            <SendRoundedIcon />
                        </IconButton>
                    </ChatbotInputMessageContainer>
                </ChatbotMessageContainer>
            </ChatbotWrapper>
        </ThemeProvider>
    );
};
